#include <EEPROM.h>
#include <Wire.h>
#include "TM1636.h"
#include <math.h>
#define BUZZER 6  // Pin bzučáku
#define TEMPSENSOR A0 // Pin senzoru teploty
#define LIGHTSENSOR A1  // Pin Fotorezistoru
#define CLOCK_ADDRESS 0x68 // I2C adresa RTC
TM1636 disp(7,8); //Deklaruje třídu pro ovládání displeje

String tempType = "C";
int maxTemp = 0;
int minTemp = 0;

void setup(){
  pinMode(BUZZER, OUTPUT);
  disp.init(); //aktivuje a vyčistí display
  Serial.begin(9600); // Otevře seriový port
  Wire.begin(); // Inicializuje knihovnu wire pro I2C komunikaci

//Vytažení kritických teplot z paměti první pozice znaménko[0==+, 1==-] druhá je už samotná teplota
  if(EEPROM.read(17) == 1){
    maxTemp = EEPROM.read(18) * -1;
    } else {
      maxTemp = EEPROM.read(18);
      }
  if(EEPROM.read(19) == 1){
    maxTemp = EEPROM.read(18) * -1;
    } else {
      maxTemp = EEPROM.read(20);
      }

      
}
void loop(){
  tempMinMax(readTemp("C")); //kontrola teploty
  while(doorOpen()){ // Když se dveře otevřou
    long openMillis = millis(); // Zápíšu si milisekundy při kterých se otevřely
    while((millis()-openMillis) < 30000 && doorOpen()){ // Čekám 30 sekund a mezi tím zobrazuji teplotu
    showTemp();
    tempMinMax(readTemp("C")); //zapsaní min max teplot
    }
    while(doorOpen()){ // Pípaní bzučáku po překročení limitu
        beep();
        tempMinMax(readTemp("C"));
      }
     disp.init(); // Zhasnutí displeje
    }
  
  
}
int readTemp(String type){ // Zjištění teploty
  int Bvalue = 3975; //Hodnota z datasheetu
  int measure = analogRead(TEMPSENSOR); //čtení hodnoti pinu
  float Rvalue = (float)(1023-measure)*10000/measure; //Odpor thermistoru
  float tempC = 1/(log(Rvalue/10000)/Bvalue+1/298.15)-273.15; //teplota v C
  float tempF = tempToF(tempC);
  if (type == "C"){
    return (int)tempC; //Vrácení v C
  }
  if(type == "F"){
      return (int)tempF; // Vrácení v F
      }
      return NULL;
}
byte decToBcd(byte val){ //převod mezi desítkovou a šestnáctkovou soustavou 
    return( (val/10*16) + (val%10) );
}

byte bcdToDec(byte val){ //převod mezi šestnáctkovou a desítkovou soustavou
    return( (val/16*10) + (val%16) );
}
void setTime(byte second, byte minute, byte hour, byte dayOfWeek, byte dayOfMonth, byte month, byte year){ //Nastavení času a datumu
    Wire.beginTransmission(CLOCK_ADDRESS); // Otevření komunikace
    Wire.write(0); // Posunutí v registru 
    Wire.write(decToBcd(second)); // Sekundy
    Wire.write(decToBcd(minute)); // Minuty
    Wire.write(decToBcd(hour)); // Hodiny
    Wire.write(decToBcd(dayOfWeek)); // Den v týdnu (Začínáme nedělí)
    Wire.write(decToBcd(dayOfMonth)); // Den
    Wire.write(decToBcd(month)); // Měsíc
    Wire.write(decToBcd(year)); // Rok (zapisujeme jen konec 20xx)
    Wire.endTransmission(); // Ukončení komunikace
}
void readTime(byte *second, byte *minute, byte *hour, byte *dayOfWeek, byte *day, byte *month, byte *year){
    Wire.beginTransmission(CLOCK_ADDRESS); //Začátek přenosu
    Wire.write(0); // Nastaví ukazatel registru na 0
    Wire.endTransmission(); // Ukončení přenosu
    Wire.requestFrom(CLOCK_ADDRESS, 7); // Vyžádání přenosu ze Slave zařízení
    *second = bcdToDec(Wire.read() & 0x7f); //Sekundy
    *minute = bcdToDec(Wire.read()); //Minuty
    *hour = bcdToDec(Wire.read() & 0x3f); //Hodiny
    *dayOfWeek = bcdToDec(Wire.read()); //Den v Týdnu
    *day = bcdToDec(Wire.read()); // Den
    *month = bcdToDec(Wire.read()); // Měsíc
    *year = bcdToDec(Wire.read()); // Rok
}
void showTemp(){ // Zobrazení teploty na displeji
    int temp = readTemp(tempType); // Přečtení teploty
    int8_t out[4]; // Výstupní pole
    int i = 2; // Pozice do které budeme zapisovat
    if(temp < 100){ // Pokud bude mít číslo jen dva znaky posuneme o pozici
      i++;
      out[2] = 18;
      }
    while(temp > 0){ // Rozdělení na jednotlivé znaky a vepsání do výstupního pole
        int d = temp / 10;
        out[i] = temp - d * 10;
        temp = d;
        i--;
      }
     if(tempType == "C"){ // Vrácení hodnoty v C
      out[3] = 12;
      }
     if(tempType == "F"){ // Vrácení hodnoty v F
      out[3] = 15;
      }
    disp.display(out); // Vypsání na display
}
boolean doorOpen(){
    if(analogRead(LIGHTSENSOR) > 300) return true; //Dveře otevřené
    if(analogRead(LIGHTSENSOR) < 300) return false; // Dveře zavřené
  }
void tempMinMax(int temp){ //Uložení teploty do paměti. vše v C
  byte second, minute, hour, dayOfWeek, day, month, year;
  byte mark = 0;
  readTime(&second, &minute, &hour, &dayOfWeek, &day, &month, &year);
  // Adresy v paměti:
  //1-8 max teplota (teplota, rok, mesíc, den, hodina, minuta, sekunda)
  //9-16 min teplota (znaménko[0==+, 1==-],teplota, rok, mesíc, den, hodina, minuta, sekunda)
  if(temp < 0){
      mark = 1;
      temp = temp * -1;
  } else {
      mark = 0;
    }
  if(temp > EEPROM.read(1)){
    EEPROM.write(1, mark);
    EEPROM.write(2, (byte)temp);
    EEPROM.write(3, year);
    EEPROM.write(4, month);
    EEPROM.write(5, day);
    EEPROM.write(6, hour);
    EEPROM.write(7, minute);
    EEPROM.write(8, second);
  }
  if(temp < EEPROM.read(9)){
    EEPROM.write(9, mark);
    EEPROM.write(10, (byte)temp);
    EEPROM.write(11, year);
    EEPROM.write(12, month);
    EEPROM.write(13, day);
    EEPROM.write(14, hour);
    EEPROM.write(15, minute);
    EEPROM.write(16, second);
  }
}
void eepromClear(){ // Vymazání paměti EEPROM
  for (int i = 0 ; i < EEPROM.length() ; i++) {
    EEPROM.write(i, 0);
  }
}
void beep(){
  digitalWrite(BUZZER, HIGH);
  delay(100);
  digitalWrite(BUZZER, LOW);
  delay(100);      
}
int tempToF(int tempC){
  return (9*tempC + 160)/5; // Převod na F
  }
void tempAllert(int temp){
  while(temp > maxTemp || temp < minTemp){
      
    }  
}
